import java.sql.Timestamp;

import org.jmock.Mock;
import org.jmock.MockObjectTestCase;

public class TimedCacheTest extends MockObjectTestCase {
	private static final Timestamp CURRENT_TIME = new Timestamp(System
			.currentTimeMillis());
	private static final Timestamp LOAD_TIME = CURRENT_TIME;
	private static final Object KEY = new Object();
	private static final Object VALUE = new Object();
	private static final Object NEW_VALUE = new Object();
	private Mock mockLoader;
	private Mock mockClock;
	private Mock mockReloadPolicy;
	private TimedCache cache;

	protected void setUp() throws Exception {
		mockLoader = mock(ObjectLoader.class);
		mockClock = mock(Clock.class);
		mockReloadPolicy = mock(ReloadPolicy.class);
		cache = new TimedCache((ObjectLoader) mockLoader.proxy(),
				(Clock) mockClock.proxy(), (ReloadPolicy) mockReloadPolicy
						.proxy());
		super.setUp();
	}

	public void testLoadsObjectThatIsNotCached() {
		mockLoader.expects(once()).method("load").with(eq(KEY)).will(
				returnValue(VALUE));
		mockClock.expects(once()).method("getCurrentTime").withNoArguments()
				.will(returnValue(CURRENT_TIME));
		mockReloadPolicy.expects(never()).method("shouldReload");
		assertSame("should load the object", VALUE, cache.lookUp(KEY));
	}

	public void testLoadsObjectIsNotCached() {
		mockLoader.expects(once()).method("load").with(eq(KEY)).will(
				returnValue(VALUE));
		mockClock.expects(exactly(2)).method("getCurrentTime")
				.withNoArguments().will(returnValue(CURRENT_TIME));
		mockReloadPolicy.expects(atLeastOnce()).method("shouldReload").with(
				eq(LOAD_TIME), eq(CURRENT_TIME)).will(returnValue(false));
		assertSame("should load the object", VALUE, cache.lookUp(KEY));
		assertSame("should get hte cached object", VALUE, cache.lookUp(KEY));
	}

	public void testCachedObjectsAreNotReloadedBeforeTimeout() {
		mockLoader.expects(once()).method("load").with(eq(KEY)).will(
				returnValue(VALUE));
		mockClock.expects(atLeastOnce()).method("getCurrentTime")
				.withNoArguments().will(returnValue(CURRENT_TIME));
		mockReloadPolicy.expects(atLeastOnce()).method("shouldReload").with(
				eq(LOAD_TIME), eq(CURRENT_TIME)).will(returnValue(false));
		assertSame("loaded object", VALUE, cache.lookUp(KEY));
		assertSame("cached object", VALUE, cache.lookUp(KEY));
	}

	public void testReloadsCachedObjectAfterTimeout() {
		mockClock.expects(atLeastOnce()).method("getCurrentTime")
				.withNoArguments().will(returnValue(CURRENT_TIME));
		mockLoader.expects(exactly(2)).method("load").with(eq(KEY)).will(
				onConsecutiveCalls(returnValue(VALUE), returnValue(NEW_VALUE)));
		mockReloadPolicy.expects(atLeastOnce()).method("shouldReload").with(
				eq(LOAD_TIME), eq(CURRENT_TIME)).will(returnValue(true));
		assertSame("should be loaded object", VALUE, cache.lookUp(KEY));
		assertSame("should be reloaded object", NEW_VALUE, cache.lookUp(KEY));
	}
}
